@@ -90,7 +90,7 @@ MathStructure Calculator::convertToMixedUnits(const MathStructure &mstruct, cons
 			}
 			for(size_t i = 0; i < units.size(); i++) {
 				Unit *ui = units[i];
-				if(ui->subtype() == SUBTYPE_ALIAS_UNIT && ((AliasUnit*) ui)->firstBaseUnit() == u  && ((AliasUnit*) ui)->firstBaseExponent() == 1) {
+				if(ui->isActive() && ui->subtype() == SUBTYPE_ALIAS_UNIT && ((AliasUnit*) ui)->firstBaseUnit() == u && ((AliasUnit*) ui)->firstBaseExponent() == 1) {
 					AliasUnit *aui = (AliasUnit*) ui;
 					int priority_i = aui->mixWithBase();
 					if(((priority_i > 0 && (!best_u || priority_i <= best_priority)) || (best_priority == 0 && priority_i == 0 && ((muc == MIXED_UNITS_CONVERSION_FORCE_INTEGER && aui->expression().find_first_not_of(NUMBERS) == string::npos) || muc == MIXED_UNITS_CONVERSION_FORCE_ALL))) && (aui->mixWithBaseMinimum() <= 1 || nr.isGreaterThanOrEqualTo(aui->mixWithBaseMinimum()))) {
@@ -168,7 +168,7 @@ MathStructure Calculator::convertToMixedUnits(const MathStructure &mstruct, cons
 				int best_priority = 0;
 				for(size_t i = 0; i < units.size(); i++) {
 					Unit *ui = units[i];
-					if(ui->subtype() == SUBTYPE_ALIAS_UNIT && ((AliasUnit*) ui)->firstBaseUnit() == u && ((AliasUnit*) ui)->firstBaseExponent() == 1) {
+					if(ui->isActive() && ui->subtype() == SUBTYPE_ALIAS_UNIT && ((AliasUnit*) ui)->firstBaseUnit() == u && ((AliasUnit*) ui)->firstBaseExponent() == 1) {
 						AliasUnit *aui = (AliasUnit*) ui;
 						int priority_i = aui->mixWithBase();
 						if(aui->expression().find("1" DIVISION) == 0 && aui->expression().find_first_not_of(NUMBERS DIVISION, 2) == string::npos && ((priority_i > 0 && (!best_u || priority_i <= best_priority)) || (best_priority == 0 && priority_i == 0 && (muc == MIXED_UNITS_CONVERSION_FORCE_INTEGER || muc == MIXED_UNITS_CONVERSION_FORCE_ALL)))) {
@@ -816,6 +816,49 @@ bool angle_convert(MathStructure &m, Unit *u, const EvaluationOptions &eo) {
 	return b;
 }
 
+int contains_angle_unit_cc(const MathStructure &m) {
+	if(m.isUnit() && m.unit()->baseUnit() == CALCULATOR->getRadUnit() && m.unit()->baseExponent() == 1) {
+		if(m.unit()->referenceName() == "turn") return 3;
+		if(m.unit() == CALCULATOR->getRadUnit()) return 1;
+		return 2;
+	}
+	int ret = 0;
+	for(size_t i = 0; i < m.size(); i++) {
+		if(!m.isFunction() || !m.function()->getArgumentDefinition(i + 1) || m.function()->getArgumentDefinition(i + 1)->type() != ARGUMENT_TYPE_ANGLE) {
+			int i_ret = contains_angle_unit_cc(m[i]);
+			if(i_ret > ret) ret = i_ret;
+		}
+	}
+	return ret;
+}
+int contains_non_angle_unit_cc(const MathStructure &m) {
+	if(m.isUnit()) return m.unit() != CALCULATOR->getRadUnit();
+	for(size_t i = 0; i < m.size(); i++) {
+		if(contains_non_angle_unit_cc(m[i])) return true;
+	}
+	return false;
+}
+bool remove_rad_unit_cc(MathStructure &m, const EvaluationOptions &eo) {
+	if(m.isUnit()) {
+		if(m.unit() == CALCULATOR->getRadUnit()) {
+			m.set(1, 1, 0, true);
+			return true;
+		}
+	} else {
+		bool b = false;
+		for(size_t i = 0; i < m.size(); i++) {
+			if((!m.isFunction() || !m.function()->getArgumentDefinition(i + 1) || m.function()->getArgumentDefinition(i + 1)->type() != ARGUMENT_TYPE_ANGLE) && remove_rad_unit_cc(m[i], eo)) {
+				b = true;
+			}
+		}
+		if(b) {
+			m.calculatesub(eo, eo, false);
+			return true;
+		}
+	}
+	return false;
+}
+
 MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, const EvaluationOptions &eo, bool always_convert, bool convert_to_mixed_units, bool transform_orig, MathStructure *parsed_struct) {
 	CompositeUnit *cu = NULL;
 	if(to_unit->subtype() == SUBTYPE_COMPOSITE_UNIT) cu = (CompositeUnit*) to_unit;
@@ -845,15 +888,28 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 			fix_to_struct(munit);
 			if(!munit.isZero()) {
 				MathStructure mstruct_new(mstruct);
-				mstruct_new *= munit;
-				PrintOptions po = message_printoptions;
-				po.negative_exponents = false;
-				munit.format(po);
-				if(munit.isMultiplication() && munit.size() >= 2) {
-					if(munit[0].isOne()) munit.delChild(1, true);
-					else if(munit[1].isOne()) munit.delChild(2, true);
+				if(mstruct_new.isComparison() || (mstruct_new.isLogicalAnd() && mstruct_new.size() > 0 && mstruct_new[0].isComparison()) || (mstruct_new.isLogicalOr() && mstruct_new.size() > 0 && (mstruct_new[0].isComparison() || (mstruct_new[0].isLogicalAnd() && mstruct_new[0].size() > 0 && mstruct_new[0][0].isComparison())))) {
+					if(mstruct_new.isLogicalOr()) {
+						for(size_t i = 0; i < mstruct_new.size(); i++) {
+							if(mstruct_new[i].isComparison()) mstruct_new[i][1] *= munit;
+							else if(mstruct_new[i].isLogicalAnd() && mstruct_new[i].size() > 0 && mstruct_new[i][0].isComparison()) mstruct_new[i][0][1] *= munit;
+						}
+					} else {
+						if(mstruct_new.isLogicalAnd()) mstruct_new[0][1] *= munit;
+						else mstruct_new[1] *= munit;
+					}
+					mstruct_new.childrenUpdated(true);
+				} else {
+					mstruct_new *= munit;
+					PrintOptions po = message_printoptions;
+					po.negative_exponents = false;
+					munit.format(po);
+					if(munit.isMultiplication() && munit.size() >= 2) {
+						if(munit[0].isOne()) munit.delChild(1, true);
+						else if(munit[1].isOne()) munit.delChild(2, true);
+					}
+					if(parsed_struct) parsed_struct->multiply(munit, true);
 				}
-				if(parsed_struct) parsed_struct->multiply(munit, true);
 				return convert(mstruct_new, to_unit, eo, always_convert, convert_to_mixed_units, false, NULL);
 			}
 		}
@@ -862,6 +918,19 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 	MathStructure mstruct_new(mstruct);
 	size_t n_messages = messages.size();
 	if(to_unit->hasNonlinearRelationTo(to_unit->baseUnit()) && to_unit->baseUnit()->subtype() == SUBTYPE_COMPOSITE_UNIT) {
+		if(mstruct_new.isComparison() || (mstruct_new.isLogicalAnd() && mstruct_new.size() > 0 && mstruct_new[0].isComparison()) || (mstruct_new.isLogicalOr() && mstruct_new.size() > 0 && (mstruct_new[0].isComparison() || (mstruct_new[0].isLogicalAnd() && mstruct_new[0].size() > 0 && mstruct_new[0][0].isComparison())))) {
+			if(mstruct_new.isLogicalOr()) {
+				for(size_t i = 0; i < mstruct_new.size(); i++) {
+					if(mstruct_new[i].isComparison()) mstruct_new[i][1].set(convert(mstruct_new[i][1], to_unit, eo, always_convert, convert_to_mixed_units, transform_orig));
+					else if(mstruct_new[i].isLogicalAnd() && mstruct_new[i].size() > 0 && mstruct_new[i][0].isComparison()) mstruct_new[i][0][1].set(convert(mstruct_new[i][0][1], to_unit, eo, always_convert, convert_to_mixed_units, transform_orig));
+				}
+			} else {
+				if(mstruct_new.isLogicalAnd()) mstruct_new[0][1].set(convert(mstruct_new[0][1], to_unit, eo, always_convert, convert_to_mixed_units, transform_orig));
+				else mstruct_new[1].set(convert(mstruct_new[1], to_unit, eo, always_convert, convert_to_mixed_units, transform_orig));
+			}
+			mstruct_new.childrenUpdated(true);
+			return mstruct_new;
+		}
 		mstruct_new = convert(mstruct, to_unit->baseUnit(), eo, always_convert, convert_to_mixed_units, transform_orig, parsed_struct);
 		mstruct_new.calculateDivide(((CompositeUnit*) to_unit->baseUnit())->generateMathStructure(false, eo.keep_prefixes), eo);
 		to_unit->convertFromBaseUnit(mstruct_new);
@@ -887,7 +956,7 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 		if(mstruct_new.size() > 0) {
 			for(size_t i = 0; i < mstruct_new.size(); i++) {
 				if(aborted()) return mstruct;
-				if(!mstruct_new.isFunction() || !mstruct_new.function()->getArgumentDefinition(i + 1) || mstruct_new.function()->getArgumentDefinition(i + 1)->type() == ARGUMENT_TYPE_FREE || (!mstruct_new.function()->getArgumentDefinition(i + 1)->tests() && mstruct_new.function()->getArgumentDefinition(i + 1)->type() != ARGUMENT_TYPE_ANGLE)) {
+				if((i != 0 || !mstruct_new.isComparison() || mstruct[0].containsType(STRUCT_UNIT) > 0) && (!mstruct_new.isFunction() || !mstruct_new.function()->getArgumentDefinition(i + 1) || mstruct_new.function()->getArgumentDefinition(i + 1)->type() == ARGUMENT_TYPE_FREE || (!mstruct_new.function()->getArgumentDefinition(i + 1)->tests() && mstruct_new.function()->getArgumentDefinition(i + 1)->type() != ARGUMENT_TYPE_ANGLE))) {
 					mstruct_new[i] = convert(mstruct_new[i], to_unit, eo, false, convert_to_mixed_units, false, NULL);
 					if(!b_changed && !mstruct_new.equals(mstruct[i], true, true)) b_changed = true;
 				}
@@ -1149,16 +1218,20 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 				mstruct_new.eval(eo2);
 				eo2.mixed_units_conversion = MIXED_UNITS_CONVERSION_NONE;
 				bool b_pos = !cu, b_neg = false;
-				if(cu || to_unit->baseUnit()->subtype() == SUBTYPE_COMPOSITE_UNIT) {
-					CompositeUnit *cu2 = cu;
-					if(cu2) b_pos = true;
-					else cu2 = (CompositeUnit*) to_unit->baseUnit();
+				CompositeUnit *cu2 = cu;
+				if(cu2) b_pos = true;
+				else if(to_unit->baseUnit()->subtype() == SUBTYPE_COMPOSITE_UNIT) cu2 = (CompositeUnit*) to_unit->baseUnit();
+				if(cu2) {
 					int exp2 = 1;
 					for(size_t i = 1; i <= cu2->countUnits(); i++) {
 						cu2->get(i, &exp2);
+						if(!cu) exp2 *= to_unit->baseExponent();
+						exp2 *= cu2->get(i)->baseExponent();
 						if(exp2 < 0) b_neg = true;
 						else b_pos = true;
 					}
+				} else {
+					b_neg = to_unit->baseExponent() < 0;
 				}
 				long int n = count_unit_powers(mstruct_new);
 				test_convert(mstruct_new, to_unit, n, b_pos, eo2);
@@ -1217,6 +1290,7 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 						exp = 1;
 					}
 				}
+				if(!b_ratio && !b_angle && contains_angle_unit_cc(mstruct_new) == 3) mstruct_new.convert(getRadUnit(), false, NULL, true, eo2);
 				if(cu) {
 					MathStructure *mstruct_cu = new MathStructure(cu->generateMathStructure(false, eo.keep_prefixes));
 					Prefix *p = NULL;
@@ -1316,9 +1390,9 @@ Unit *Calculator::findMatchingUnit(const MathStructure &mstruct) {
 				}
 				for(size_t i = 0; i < units.size(); i++) {
 					Unit *u = units[i];
-					if(u->subtype() == SUBTYPE_ALIAS_UNIT && u->baseUnit() == u_base && ((AliasUnit*) u)->baseExponent() == exp) {
+					if(u->isActive() && u->subtype() == SUBTYPE_ALIAS_UNIT && u->baseUnit() == u_base && ((AliasUnit*) u)->baseExponent() == exp) {
 						return u;
-					} else if(u->subtype() == SUBTYPE_COMPOSITE_UNIT && ((CompositeUnit*) u)->countUnits() == 1 && ((CompositeUnit*) u)->get(1, &exp2)->baseUnit() == u_base && exp == ((CompositeUnit*) u)->get(1)->baseExponent(exp2)) {
+					} else if(u->isActive() && u->subtype() == SUBTYPE_COMPOSITE_UNIT && ((CompositeUnit*) u)->countUnits() == 1 && ((CompositeUnit*) u)->get(1, &exp2)->baseUnit() == u_base && exp == ((CompositeUnit*) u)->get(1)->baseExponent(exp2)) {
 						return u;
 					}
 				}
@@ -1361,10 +1435,9 @@ Unit *Calculator::findMatchingUnit(const MathStructure &mstruct) {
 				if(exp == 1) return u_base;
 				for(size_t i = 0; i < units.size(); i++) {
 					Unit *u = units[i];
-					if(u->subtype() == SUBTYPE_ALIAS_UNIT && u->baseUnit() == u_base && ((AliasUnit*) u)->baseExponent() == exp) {
+					if(u->isActive() && u->subtype() == SUBTYPE_ALIAS_UNIT && u->baseUnit() == u_base && ((AliasUnit*) u)->baseExponent() == exp) {
 						return u;
-					} else if(
-						u->subtype() == SUBTYPE_COMPOSITE_UNIT && ((CompositeUnit*) u)->countUnits() == 1 && ((CompositeUnit*) u)->get(1, &exp2)->baseUnit() == u_base && exp == ((CompositeUnit*) u)->get(1)->baseExponent(exp2)) {
+					} else if(u->isActive() && u->subtype() == SUBTYPE_COMPOSITE_UNIT && ((CompositeUnit*) u)->countUnits() == 1 && ((CompositeUnit*) u)->get(1, &exp2)->baseUnit() == u_base && exp == ((CompositeUnit*) u)->get(1)->baseExponent(exp2)) {
 						return u;
 					}
 				}
@@ -1372,7 +1445,7 @@ Unit *Calculator::findMatchingUnit(const MathStructure &mstruct) {
 			if(cu->countUnits() > 1) {
 				for(size_t i = 0; i < units.size(); i++) {
 					Unit *u = units[i];
-					if(u->subtype() == SUBTYPE_COMPOSITE_UNIT) {
+					if(u->isActive() && u->subtype() == SUBTYPE_COMPOSITE_UNIT) {
 						if(((CompositeUnit*) u)->countUnits() == cu->countUnits()) {
 							bool b = true;
 							for(size_t i2 = 1; i2 <= cu->countUnits(); i2++) {
@@ -1486,7 +1559,7 @@ Unit *Calculator::getOptimalUnit(Unit *u, bool allow_only_div, bool convert_to_l
 			Unit *best_u = NULL;
 			if(cu->countUnits() > 1) {
 				for(size_t i = 0; i < units.size(); i++) {
-					if(units[i]->subtype() == SUBTYPE_COMPOSITE_UNIT && !units[i]->isHidden() && units[i]->isSIUnit()) {
+					if(units[i]->isActive() && units[i]->subtype() == SUBTYPE_COMPOSITE_UNIT && !units[i]->isHidden() && units[i]->isSIUnit()) {
 						CompositeUnit *cu2 = (CompositeUnit*) units[i];
 						if(cu == cu2 && !cu2->isHidden()) {
 							points = max_points - 1;
@@ -1507,7 +1580,7 @@ Unit *Calculator::getOptimalUnit(Unit *u, bool allow_only_div, bool convert_to_l
 					}
 				}
 				for(size_t i = 0; i < units.size() && !best_u; i++) {
-					if(units[i]->subtype() == SUBTYPE_COMPOSITE_UNIT && !units[i]->isHidden() && units[i]->isSIUnit()) {
+					if(units[i]->isActive() && units[i]->subtype() == SUBTYPE_COMPOSITE_UNIT && !units[i]->isHidden() && units[i]->isSIUnit()) {
 						CompositeUnit *cu2 = (CompositeUnit*) units[i];
 						if(cu2->countUnits() <= cu->countUnits()) {
 							for(size_t i2 = 1; i2 <= cu2->countUnits(); i2++) {
@@ -1559,6 +1632,7 @@ Unit *Calculator::getOptimalUnit(Unit *u, bool allow_only_div, bool convert_to_l
 			AliasUnit *au;
 			for(size_t i = 0; i < units.size(); i++) {
 				u2 = units[i];
+				if(!u2->isActive()) continue;
 				if(u2->subtype() == SUBTYPE_BASE_UNIT && (points == 0 || (points == 1 && minus))) {
 					for(size_t i2 = 1; i2 <= cu->countUnits(); i2++) {
 						if(cu->get(i2, &exp)->baseUnit() == u2 && !cu->get(i2)->hasNonlinearRelationTo(u2)) {
